/* the lexical scanner for NUALGOL  */
%{

#include <stdio.h>
#include "algol.h"

int global_store = 0;     /* normally not anything */

%}
QUOTE		[\"]
WS		[ \t\f\n\r]+
OPTWS		[ \t\f\n\r]*
BLANK		[ \t]{1}
ID_TYPE		[A-Za-z][A-Za-z_$0-9]*
DIGIT		[0-9]
DIGITS		[0-9]*
DOT		[\.]
OCTAL		[0-7]*
HEX		[0-9A-F]
%%
"and"		{ return(AND); }
"array"		{ global_store = ARRAY; return(ARRAY); }
"begin"		{ return(BEGIN_SYM); }
"boolean"	{ global_type = BOOLEAN; return(BOOLEAN); }
"case"		{ return(CASE); }
"do"		{ return(DO); }
"else"		{ return(ELSE); }
"end"		{ return(END_SYM); }
"endcase"	{ return(ENDCASE); }
"endstr"	{ return(ENDSTRUCTURE); }
"endstruct"	{ return(ENDSTRUCTURE); }
"endstructure"	{ return(ENDSTRUCTURE); }
"external"	{ return(EXTERNAL); }
"false"		{ return(FALSE); }
"for"		{ return(FOR); }
"function"	{ global_type = FUNCTION; return(FUNCTION);}
"go"		{ return(GO); }
"goto"		{ return(GOTO); }
"if"		{ return(IF); }
"integer"	{ global_type = INTEGER; return(INTEGER);}
"literal"	{ global_type = LITERAL; return(LITERAL);}
"not"		{ return(NOT); }
"or"		{ return(OR); }
"own"		{ global_store = OWN; return(OWN);}
"proc"		{ return(PROCEDURE); }
"procedure"	{ return(PROCEDURE); }
"pointer"	{ global_type = POINTER; return(POINTER);}
"real"		{ global_type = REAL; return(REAL);}
:"return"	{ return(RETURN); }
"step"		{ return(STEP); }
"string"	{ global_type = STRING; return(STRING);}
"struct"	{ return(STRUCTURE); }
"structure"	{ return(STRUCTURE); }
"to"		{ return(TO); }
"then"		{ return(THEN); }
"true"		{ return(TRUE); }
"type"		{ return(TYPE); }
"union"		{ global_type = UNION; return(UNION);}
"until"		{ return(UNTIL); }
"while"		{ return(WHILE); }
":="		{ return(ARROW); }
"+"		{ return(ADD_OP); }
"||"		{ return(CONCAT); }
":"		{ return(COLON); }
","		{ return(COMMA); }
{DIGITS}	{ return(NUMERIC_CONSTANT); }
"*"		{ return(MUL_OP); }
">"		{ return(REL_OP); }
";"		{ global_type = ID; global_store = 0; return(SEMICOLON);}
"**"		{ return(STARSTAR); }
"="		{ return(LOG_OP); }
"//"		{ return(MOD); }
"(*"		{ return(LEFT_MARK); }
"*)"		{ return(RIGHT_MARK); }
"("		{ return(LEFT_PAREN); }
")"		{ return(RIGHT_PAREN); }
"@"		{ return(ATSIGN); }
{QUOTE}{QUOTE}	{ return(QUOTEQUOTE); }
{QUOTE}.{QUOTE}	{ return(STRING_CONSTANT); }
"\["		{ return(LEFT_SQUARE);
"\]"		{ return(RIGHT_SQUARE);
{ID_TYPE}	{ return(symbol(yytext[0])); }
%%
int last_sym;
char filename[256];  /* this is the size of a path  4096*/;

int
yywrap()
{
 yyin = fopen(filename,"r+");
return 1;
};


char *yytext;

struct  sym_table {	/* global symbol table */
	char * var_name ;	/* symbol name */
	long address ;		/* definition address */
	int type ;		/* int,real,string, etc */
	int value ;		/* any known value or 0 */
	int indexes ;		/* any array indexes or 0*/
	}; 

#include <stdio.h>
#include <stdlib.h>

struct Tree_block
{
struct sym_table *sym_table;
double operand;
char type, operat; 
struct Tree_block *left_child, *right_child;
};


struct Stack_block
{
struct Tree_block  *tree_address;
struct Stack_block *next_node;
};

void * push( struct Tree_block *address, struct Stack_block *head );

struct Tree_block *pop(struct Stack_block *head);


int i;
double number; 
char optr, tp;
struct Tree_block *current, *end; 
struct Stack_block *head, *tail;

init()
{
head=(struct Stack_block *)calloc(1,sizeof(struct Stack_block));
tail=(struct Stack_block *)calloc(1,sizeof(struct Stack_block));
end=(struct Tree_block  *)calloc(1,sizeof(struct Tree_block));

head->next_node=tail;
last_sym = 1;
};

symbol(char *name)
{
current=(struct Tree_block *)calloc(1, sizeof(struct Tree_block));
for (i=1; i<=last_sym; i++)
{    
current->right_child= head;
current->left_child = push(&current,&head);
push(&current, &head);
}

}; /* end of main  */
 
void *
push(struct Tree_block *address, struct Stack_block *head) 
{
struct Stack_block *new_node;

new_node=(struct Stack_block *)calloc(1,sizeof(struct Stack_block));

new_node->tree_address=address;
new_node->next_node=head->next_node;
head->next_node=new_node;
}

struct Tree_block *pop( struct Stack_block *head)
{
struct Stack_block dummy;
struct Tree_block address;

address=*(head->next_node)->tree_address;
dummy=*head->next_node;
head->next_node=(head->next_node)->next_node;
free (&dummy);
return(&address);
}

        main( argc, argv )
        int argc;
        char **argv;
{
        char gp;

        if ( argc < 2 )
        {
        fprintf(stdout, " ltest  filename \n");
        exit(0);  /* MWC exit is zero == OK */
        }
           strcpy( filename, argv[1]);
        gp =   yywrap();
       printf(" filename = %s \n");

        while (gp =  yylex() )

         printf("%d  \"%s\"\n", gp, yytext);

        printf(" done\n");
};


/* joe widows  */
