/* the lexical scanner for NUALGOL  */
%{
#include <stdio.h>
#include "algol.h"
static char copyright[] =
" @(#) Copyright (c) 1985 Joe Widows, Arlington Texas.\n";
static char CR_continuation[] = "@(#) All rights reserved.\n";
static char manufacture[] = " January, 28, 2004 \n";

int global_type = ID;     /* start out with this state */
int global_store = 0;     /* normally not anything */

%}
QUOTE		[\"]
ID_TYPE		[A-Za-z][A-Za-z_$0-9]*
DIGIT		[0-9]
DIGITS		[0-9]*
DOT		[\.]
OCTAL		[0-7]*
HEX		[0-9A-F]
%%
"and"		{ return(AND); }
"array"		{ global_store = ARRAY; return(ARRAY); }
"begin"		{ return(BEGIN_SYM); }
"boolean"	{ global_type = BOOLEAN; return(BOOLEAN); }
"case"		{ return(CASE); }
"do"		{ return(DO); }
"else"		{ return(ELSE); }
"end"		{ return(END_SYM); }
"endcase"	{ return(ENDCASE); }
"endstr"	{ return(ENDSTRUCTURE); }
"endstruct"	{ return(ENDSTRUCTURE); }
"endstructure"	{ return(ENDSTRUCTURE); }
"external"	{ return(EXTERNAL); }
"false"		{ return(FALSE); }
"for"		{ return(FOR); }
"function"	{ global_type = FUNCTION; return(FUNCTION);}
"go"		{ return(GO); }
"goto"		{ return(GOTO); }
"if"		{ return(IF); }
"integer"	{ global_type = INTEGER; return(INTEGER);}
"literal"	{ global_type = LITERAL; return(LITERAL);}
"not"		{ return(NOT); }
"or"		{ return(OR); }
"own"		{ global_store = OWN; return(OWN);}
"proc"		{ return(PROCEDURE); }
"procedure"	{ return(PROCEDURE); }
"pointer"	{ global_type = POINTER; return(POINTER);}
"real"		{ global_type = REAL; return(REAL);}
"return"	{ return(RETURN); }
"step"		{ return(STEP); }
"string"	{ global_type = STRING; return(STRING);}
"struct"	{ return(STRUCTURE); }
"structure"	{ return(STRUCTURE); }
"to"		{ return(TO); }
"then"		{ return(THEN); }
"true"		{ return(TRUE); }
"type"		{ return(TYPE); }
"union"		{ global_type = UNION; return(UNION);}
"until"		{ return(UNTIL); }
"while"		{ return(WHILE); }
":="		{ return(ARROW); }
"+"		{ return(ADD_OP); }
"||"		{ return(CONCAT); }
":"		{ return(COLON); }
","		{ return(COMMA); }
{DIGITS}	{ return(NUMERIC_CONSTANT); }
"*"		{ return(MUL_OP); }
">"		{ return(REL_OP); }
";"		{ global_type = ID; global_store = 0; return(SEMICOLON);}
"**"		{ return(STARSTAR); }
"="		{ return(LOG_OP); }
"\/\*.\*\/"       { /* comment */ } 
"//"		{ return(MOD); }
"(*"		{ return(LEFT_MARK); }
"*)"		{ return(RIGHT_MARK); }
"("		{ return(LEFT_PAREN); }
")"		{ return(RIGHT_PAREN); }
"@"		{ return(ATSIGN); }
{QUOTE}{QUOTE}	{ return(QUOTEQUOTE); }
{QUOTE}.{QUOTE}	{ return(STRING_CONSTANT); }
[A-Za-z][A-Za-z_$0-9]*	{ return (symbol(yytext[0]) );  }
%%
/* this was a NEW facet.  People have instutionalized the I/O mechanism */

extern int yyparse();

void yyerror( const char * msg)
{
        printf(stdout,msg);
}
FILE * ftable;

char filename[256];
yywrap()
{
 yyin = fopen(filename,"r");
 return 1;
}

struct  sym_table {	/* global symbol table */
	char * var_name ;	/* symbol name */
	long address ;		/* definition address */
	int ltype ;		/* int,real,string, etc */
	int value ;		/* any known value or 0 */
	int indexes ;		/* any array indexes or 0*/
	}; 
int last_sym=0;
int symbol( name )
	char *name;
	{
	struct sym_table *symbol_table[100];
	int test;
	int i;

	for (i=0; i++; i=last_sym) {
	test = strcmp((struct sym_table *)symbol_table[i]->var_name,name);
	if (test = 0)  return( (struct sym_table *)symbol_table[i]->ltype );
	else {
	last_sym++;
(struct sym_table *)symbol_table[last_sym] = (struct sym_table *)calloc(1, sizeof(struct sym_table));
	strcpy((struct sym_table *)symbol_table[last_sym]->var_name, name);
	(struct sym_table *)symbol_table[last_sym]->ltype = global_type;
	return(ID);
		};
	return(-1); /* if we get here there is a real problem */
            }
       /* this was written with flex and yacc on an atari 1040 st */ 
       /* in 1985, it worked under MWC then! the linker worked.  */
}/* end of program */
   FILE * ftable;
	main( argc, argv )
	int argc;
	char **argv;
{
	 int gp;
	if ( argc < 2 )
	{
	fprintf(stdout, " ltest  filename \n");
	exit(0);  /* MWC exit is zero == OK */
	}
           strcpy(filename,argv[1]);
             gp = yywrap(); /* open the file */
       
   ftable = fopen("mytest.lst","ra");

/*
 * Put out a header line at the beginning of the 'table' file.
 */
printf("/*\n * Compiled by ALGOL (ATARI ST-MWC) from \"%s\" */\n",
        filename);
fprintf(stdout,"/*\n * Compiled by ALGOL (ATARI ST-MWC) from \"%s\" */\n",
             filename);

 while (gp  != 0)  {

         gp = yylex();
         printf(" ytext = %s \n",yytext);

         printf(" gp = %d \n",gp);
        }
        
	  printf(" done\n");	
}
